#!/usr/bin/env python

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base import Script

Script.setUsageMessage("""
Get statistics for datasets corresponding to the dataset name (may contain wild card)
if no dataset is specified it gives the list of available datasets
Usage:
   %s <dataset>

""" % Script.scriptName)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


def print_dataset_list(dataset_list):
  dataset_list.sort()
  print('\nAvailable datasets are:\n')
  for dataset_name in dataset_list:
    gLogger.notice(dataset_name)
  gLogger.notice('\n')


def get_list_of_datasets(tag=''):
  fc = FileCatalogClient()
  dataset_tag = '*%s*' % tag
  res = fc.getDatasets(dataset_tag)
  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  dataset_list = list()
  for dataset_name in dataset_dict['Successful'][dataset_tag].keys():
    dataset_list.append(dataset_name)
  return dataset_list


def get_dataset_info(dataset_name):
  fc = FileCatalogClient()
  res = fc.getDatasets(dataset_name)
  if not res['OK']:
    gLogger.error("Failed to get datasets")
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  res = dataset_dict['Successful'][dataset_name][dataset_name]
  number_of_files = res['NumberOfFiles']
  meta_query = res['MetaQuery']
  total_size = res['TotalSize']
  return (dataset_name, number_of_files, total_size, meta_query)


#########################################################
def main():
  argss = Script.getPositionalArgs()

  if len(argss) == 0:
    dataset_list = get_list_of_datasets()
    print_dataset_list(dataset_list)
    DIRAC.exit()
  elif len(argss) == 1:
    dataset_name = argss[0]
    dataset_list = list()
    if dataset_name.find('*') > 0:
      dataset_list = get_list_of_datasets(dataset_name)
    else:
      dataset_list.append(dataset_name)
    print_dataset_list(dataset_list)

  # Results
  gLogger.notice('Datasets details')
  if len(dataset_list) == 1:
    name, n_files, size, mq = get_dataset_info(dataset_list[0])
    gLogger.notice('MetaQuery')
    gLogger.notice(str(mq))
  gLogger.notice('|_. Name |_. N files |_. Size(TB) |')
  dataset_list.sort()
  total_size = 0.
  total_n_files = 0
  for dataset_name in dataset_list:
    name, n_files, size, mq = get_dataset_info(dataset_name)
    # # convert total size in TB
    size_TB = size / 1e12
    gLogger.notice('|%s|%d|%.2f|' % (name, n_files, size_TB))
    total_size += size_TB
    total_n_files += n_files
  gLogger.notice('| Total | %d | %.2f |' % (total_n_files, total_size))
  DIRAC.exit()

if __name__ == '__main__':
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

import json

import DIRAC
from DIRAC.Core.Base import Script
from DIRAC import gLogger


Script.setUsageMessage("""
Add a dataset from a query specified in a json file, e.g.:

{"MCCampaign': "PROD5b",
"particle": "proton",
"site": "Paranal"}

Usage:
   %s <datasetName> <json file with dataset query dict>
""" % Script.scriptName)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

def main():
  fc = FileCatalogClient()
  argss = Script.getPositionalArgs()
  if len(argss) == 2:
    datasetName = argss[0]
    inputJson = argss[1]
  else:
    Script.showHelp()

  # read the meta data query from json file
  f = open(inputJson)
  MDdict = json.load(f)
  f.close()

  datasetPath = '/' + datasetName
  res = (fc.addDataset({datasetPath: MDdict}))

  if not res['OK']:
    gLogger.error("Failed to add dataset %s: %s" % (datasetName, res['Message']))
    DIRAC.exit(-1)

  if datasetName in res['Value']['Failed']:
    gLogger.error("Failed to add dataset %s: %s" % (datasetName, res['Value']['Failed'][datasetName]))
    DIRAC.exit(-1)

  gLogger.notice("Successfully added dataset", datasetName)
  DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Base import Script

Script.setUsageMessage("""
Update a given dataset

Usage:
   %s <dataset>
""" % Script.scriptName)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

def main():
    argss = Script.getPositionalArgs()

    fc = FileCatalogClient()

    if len(argss) > 0:
      datasetName = argss[0]
    else:
      Script.showHelp()

    result = fc.updateDataset(datasetName)

    if not result['OK']:
      gLogger.error("Failed to update dataset %s: %s" % (datasetName, result['Message']))
      DIRAC.exit(-1)
    else:
      gLogger.notice("Successfully updated dataset", datasetName)
      DIRAC.exit()

if __name__ == '__main__':
  main()



#!/usr/bin/env python

"""
  Get the files attached to a transformation and optionally update the file status
  Extension of dirac-transformation-get-files with additional options:
  --FileStatus : selection based on the FileStatus
  --TaskStatus : selection based on the TaskStatus
  --setFileStatus : update the FileStatus of the selected files to a new Status
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

from DIRAC.Core.Base.Script import parseCommandLine
parseCommandLine()

def main():

  Script.registerSwitch('', 'TransID=', '   transformation ID')
  Script.registerSwitch('', 'FileStatus=', '    file status')
  Script.registerSwitch('', 'TaskStatus=', '    task status')
  Script.registerSwitch('', 'setFileStatus=', '    new file status')

  transID = None
  fileStatus = None
  taskStatus = None
  newFileStatus = None

  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'TransID':
      try:
        transID = int(switch[1])
      except Exception:
        gLogger.fatal('Invalid transID', switch[1])
    elif switch[0] == 'FileStatus':
      fileStatus = switch[1].capitalize()
    elif switch[0] == 'TaskStatus':
      taskStatus = switch[1].capitalize()
    elif switch[0] == 'setFileStatus':
      newFileStatus = switch[1].capitalize()


  if not transID:
    Script.showHelp(exitCode=2)

  tc = TransformationClient()

  condDict = {'TransformationID': transID}

  if fileStatus:
    condDict.update({'Status': fileStatus})

  if taskStatus:
    res = tc.getTransformationTasks({'TransformationID': transID, 'ExternalStatus': taskStatus})

    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(2)

    if len(res['Value']) == 0:
      gLogger.notice("No tasks selected for transformation %s with status %s" % (transID, taskStatus))

    taskIDs = []
    for task in res['Value']:
      taskIDs.append(task["TaskID"])

    condDict.update({'TaskID': taskIDs})

  res = tc.getTransformationFiles(condDict)

  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(2)

  if len(res['Value']) == 0:
    gLogger.notice("No files selected for transformation %s with status %s" % (transID, fileStatus))

  transFiles = []
  for transfile in res['Value']:
    transFiles.append(transfile['LFN'])
    gLogger.notice(transfile['LFN'])

  if newFileStatus:
    res = tc.setFileStatusForTransformation(transID, newLFNsStatus=newFileStatus, lfns=transFiles)
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(2)
    gLogger.notice(" %d files set to %s status" % (len(transFiles), newFileStatus))

if __name__ == "__main__":
  main()


#!/usr/bin/env python
'''
Move a dataset distributed on a number of production SEs
('DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk','LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk', 'LANCASTER-Disk', 'POLGRID-Disk')
to a given SE to be passed as an argument, e.g. CC-IN2P3-Tape

    J. Bregeon, L. Arrabito November 2020
    bregeon@in2p3.fr, arrabito@in2p3.fr
'''

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base import Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.TransformationSystem.Utilities.ReplicationTransformation import createDataTransformation


Script.setUsageMessage(
  '\n'.join(
      [
          __doc__.split('\n')[1],
          'Usage:',
          '  %s <dataset name> <dest SE>' %
          Script.scriptName,
          'Optional arguments:',
          '  <group size>: size of the transformation (default=1)',
          '  <tag>: tag added to the transformation name',
          '\n\ne.g: %s Prod4_Paranal_gamma_North_20deg_SSTOnly_MC0 CC-IN2P3-Tape 100 v1' %
          Script.scriptName,
      ]))

Script.parseCommandLine(ignoreErrors=True)

def get_dataset_info(dataset_name):
  """ Return essential dataset information
      Name, number of files, total size and meta query
  """
  fc = FileCatalogClient()
  res = fc.getDatasets(dataset_name)
  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  res = dataset_dict['Successful'][dataset_name][dataset_name]
  number_of_files = res['NumberOfFiles']
  meta_query = res['MetaQuery']
  total_size = res['TotalSize']
  return (dataset_name, number_of_files, total_size, meta_query)


#########################################################
def main():
  argss = Script.getPositionalArgs()
  if len(argss) < 2:
    Script.showHelp()
  dataset_name = argss[0]
  dest_se = argss[1]
  extra_tag = ""
  group_size = 1
  if len(argss) > 2:
    group_size = argss[2]
  if len(argss) == 4:
    extra_tag = '_' + argss[3]

  # Check input data set information
  name, n_files, size, meta_query = get_dataset_info(dataset_name)
  gLogger.notice('Found dataset %s with %d files.' % (name, n_files))
  gLogger.notice(meta_query)
  # choose a metaKey
  for k,v in meta_query.items():
    meta_key = k
    meta_value = v
  tag = dataset_name + extra_tag

  do_it = True
  # To do: replace hard coded SE with SE read from CS
  source_se = ['DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk',
             'LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk', 'POLGRID-Disk', 'LANCASTER-Disk']

  # create Transformation
  res = createDataTransformation(flavour='Moving',
                                     targetSE=dest_se,
                                     sourceSE=source_se,
                                     metaKey=meta_key,
                                     metaValue=meta_value,
                                     extraData=meta_query,
                                     extraname=tag,
                                     groupSize=int(group_size),
                                     plugin='Broadcast',
                                     tGroup=None,
                                     tBody=None,
                                     enable=do_it,
                                     )
  if not res['OK']:
    gLogger.error(res["Message"])
    DIRAC.exit(-1)

  DIRAC.exit()

#########################################################
if __name__ == '__main__':
  main()


#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
import os
from multiprocessing import Pool

# DIRAC imports
from DIRAC.Core.Base import Script

Script.setUsageMessage("""
Bulk upload of a list of local files from the current directory to a Storage Element
Usage:
   %s <ascii file with lfn list> <SE>
""" % Script.scriptName)

Script.parseCommandLine(ignoreErrors=True)

args = Script.getPositionalArgs()
if len(args) > 1:
  infile = args[0]
  SE = args[1]
else:
  Script.showHelp()

from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import read_lfns_from_file

def main():
  infileList = read_lfns_from_file(infile)
  p = Pool(10)
  p.map(addfile, infileList)

def addfile(lfn):
  dirac = Dirac()
  res = dirac.addFile(lfn, os.path.basename(lfn), SE)
  if not res['OK']:
    gLogger.error('Error uploading file', lfn)
    return res['Message']

if __name__ == '__main__':
  main()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script

Script.setUsageMessage("""
Bulk get replicas from a list of lfns
Usage:
   %s <ascii file with lfn list>

""" % Script.scriptName)

Script.parseCommandLine(ignoreErrors=True)

from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from CTADIRAC.Core.Utilities.tool_box import read_lfns_from_file

fc = FileCatalog()

def main():
  args = Script.getPositionalArgs()
  if len(args) > 0:
    infile = args[0]
  else:
    Script.showHelp()

  infileList = read_lfns_from_file(infile)
  p = Pool(1)
  p.map(getReplicas, infileList)

def getReplicas(lfn):
  res = fc.getReplicas(lfn)
  if not res['OK']:
    gLogger.error('Failed to get replicas for lfn', lfn)
    return res['Message']
  if lfn in res['Value']['Successful']:
    gLogger.notice(res['Value']['Successful'][lfn])
  else:
    res = DIRAC.S_ERROR('Failed to get replicas for lfn %s' % lfn)
    gLogger.error(res['Message'])
    return res

if __name__ == '__main__':
  main()



""""

"""

def extension_metadata():
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "CTA": "dips://dcta-servers02.pic.es:9135/Configuration/Server",
          "CTA-cert": "dips://ccdcta-cert.in2p3.fr:9135/Configuration/Server",
      },
      "default_setup": "CTA",
  }




"""
  Simple Wrapper on the Job class to run ctapipe stage 1 merging tool

  Nov 19th 2020 - J. Bregeon
                  bregeon@in2p3.fr
"""

__RCSID__ = "$Id$"

# generic imports
import json

# DIRAC imports
from CTADIRAC.Interfaces.API.Prod5Stage1Job import Prod5Stage1Job


class Prod5Stage1MergeJob(Prod5Stage1Job):
  """ Job extension class for ctapipe stage1 processing,
  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Prod5Stage1Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('ctapipe_stage1')
    self.prog_name = 'ctapipe-stage1-merge'
    self.group_size = 5

  def set_meta_data(self, dl1_md):
    """ Set DL1 meta data

    Parameters:
    dl1_md -- metadata dictionary from the telescope simulation
    """
    # # Set evndisp directory meta data
    self.metadata['array_layout'] = dl1_md['array_layout']
    self.metadata['site'] = dl1_md['site']
    self.metadata['particle'] = dl1_md['particle']

    try:
      phiP = dl1_md['phiP']['=']
    except BaseException:
      phiP = dl1_md['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = dl1_md['thetaP']['=']
    except BaseException:
      thetaP = dl1_md['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id
    self.metadata['merged'] = 1

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    i_step = 0
    # step 1 -- debug
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

      env_step = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      env_step['Value']['name'] = 'Step%i_Env' % i_step
      env_step['Value']['descr_short'] = 'Dump environment'
      i_step += 1

    # step 2
    sw_step = self.setExecutable('cta-prod-setup-software',
                                 arguments='-p %s -v %s -a stage1 -g %s' %
                                 (self.package, self.version, self.compiler),
                                 logFile='SetupSoftware_Log.txt')
    sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    sw_step['Value']['descr_short'] = 'Setup software'
    i_step += 1

    # step 3 verify input data size
    # arguments are nbFiles=0 (not used) and fileSize=1000kB
    eiv_step = self.setExecutable('cta-prod-verifysteps',
                                  arguments="generic %d 1000 '*.h5'" % self.group_size,
                                  logFile='Verify_DL1_Inputs_Log.txt')

    eiv_step['Value']['name'] = 'Step%i Verify DL1 input files' % i_step
    eiv_step['Value']['descr_short'] = 'Verify DL1 Inputs'
    i_step += 1

    # step 4 run EventDisplay
    ev_step = self.setExecutable('./dirac_run_stage1_merge',
                                 logFile='ctapipe_stage1_merge_Log.txt')
    ev_step['Value']['name'] = 'Step%i_ctapipe_stage1_merge' % i_step
    ev_step['Value']['descr_short'] = 'Run ctapipe stage 1 merge'
    i_step += 1

    # step 5 set meta data and register Data
    meta_data_json = json.dumps(self.metadata)
    file_meta_data_json = json.dumps(self.file_meta_data)

    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                               self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int', 'merged': 'int'}
    meta_data_field_json = json.dumps(meta_data_field)

    # register Data
    data_output_pattern = './Data/*.h5'  # %self.output_data_level
    scripts = '../CTADIRAC/Core/scripts/'
    dm_step = self.setExecutable(scripts + 'cta-prod-managedata.py',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (meta_data_json, meta_data_field_json,
                                  file_meta_data_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # step 7 failover step
    failover_step = self.setExecutable('/bin/ls -l',
                                       modulesList=['Script', 'FailoverRequest'])
    failover_step['Value']['name'] = 'Step%s_Failover' % i_step
    failover_step['Value']['descr_short'] = 'Tag files as unused if job failed'
    i_step += 1

    # Step 8 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      i_step += 1
